#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
# Barcerlona Supercomputing Center - Centro Nacional de Supercomputacion
#        Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)
#             -
#             -
#

import sys, logging, subprocess
from dqc_wrapper.utils.commons import (initiate_logging, dir_or_file,
                                       list_files_folder)
from cdo import Cdo
from subprocess import Popen


def return_extension(file_in):
    dict_of_extensions = {
        'grib':'GRIB',
        'nc':'NetCDF',
        'zip':'Zipped'
    }
    
    ext = file_in.split('.')[-1]
    string = "%s extension (.%s)" %(dict_of_extensions[ext],ext)
    
    return string

class FileFormat:

    def __init__(self, args):
        self.path = args.input
        self.files = dir_or_file(self.path)
        self.gridded_files = ''
        self.other_files = ''
        self.formats1 = ''
        self.formats2 = ''

    def cdo_return_format(self):
        """This function uses the CDO module, with command sinfon to retrieve 
        file format
        """
        CDO = Cdo()
        files_format = list()
        f = lambda x: 'File format' in x

        for item in self.gridded_files:
            file_format = list(filter(f, CDO.sinfon(input=item)))[0]
            out0 = file_format.replace('File format:','').strip()
            output = ": ".join([return_extension(item),out0])
            files_format.append(output)
        
            if any( "File format : GRIB" in s for s in files_format ) :
                 files_format.append("Warning: grib1 being deprecated, the recommendation is to use grib2")
            if any( "File format : NetCDF2" in s for s in files_format ) :
                 files_format = [ s.replace("NetCDF2","NetCDF3 64-bit offset") for s in files_format ]
        return set(files_format)

    def file_return_format(self):
        # This function uses the file function from bash to check format
        
        files_format = list()
        for item in self.other_files:
            cmd0 = "file -b %s" %item

            p0 = Popen(
                cmd0, shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT
                )
            
            out0, out1 = p0.communicate()
            output = ": ".join([return_extension(item),str(out0)])

            if "\n" in output:
                output = output.replace("\n",'')

            if "Zipped extension" in output:
                import re

                cmd1 = "unzip -l %s" %item

                p1 = Popen(
                    cmd1, shell=True,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.STDOUT
                    )

                out1, _ = p1.communicate()

                additional_ext = list()

                for item in list(re.findall(r'[.]\w+',str(out1))):

                    if not item.endswith('zip'):
                        additional_ext.append(item)

                additional_ext = sorted(additional_ext)
                additional_ext = "(" + ', '.join(additional_ext) + ")"

                output += " " + additional_ext
            
            files_format.append(output)


        return set(files_format)

    def merge_dictionaries(self):
        x, y = self.formats1, self.formats2
        z = x.copy()
        z.update(y)
        return z

    def main(self):
        try:
            files = self.files
        except:
            files = list_files_folder(self.path)
#        print(files)
        #unique_extensions = set(files)

        initiate_logging("", 'LOG_TEST00_FILE_FORMAT', self.path)
        files_dat1, files_dat2 = list(), list()
        for item in files:
            if item.endswith(('.grib', 'grb', '.nc')):
                files_dat1.append(item)
            else:
                files_dat2.append(item)
        
        if files_dat1:
            self.gridded_files = files_dat1
            self.formats1 = self.cdo_return_format()
        
        if files_dat2:
            self.other_files = files_dat2
            self.formats2 = self.file_return_format()

        if self.formats2 != '' and self.formats1 != '':
            unique_formats = self.merge_dictionaries()
        
        elif self.formats2 == '' and self.formats1 != '':
            unique_formats = self.formats1
        
        elif self.formats2 != '' and self.formats1 == '':
            unique_formats = self.formats2
        else:
            raise Exception("No files found with grb, grib or nc extension,"
                            " exiting")

        string = '\n'
        unique_formats = sorted(unique_formats)
        for item in unique_formats:
            string += '\t%s\n' %item

        logging.info(string)

        return string

if __name__ == "__main__":
    sys.exit()
