#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
# Barcerlona Supercomputing Center - Centro Nacional de Supercomputacion
#        Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)
#             -
#             -
#

import sys, logging, subprocess
from dqc_wrapper.utils.commons import initiate_logging, dir_or_file
import dqc_wrapper.utils.grib_checker.grib_cck as gcck

class StandardComplianceTest:

    def __init__(self,path,type=None):
        self.type = type
        self.path = path
        self.files = dir_or_file(self.path)
        self.gridded_files = ''
        self.other_files = ''
        self.formats1 = ''
        self.formats2 = ''


    def run_cfchecker(self,item):
        """RUN CFCHECKER FOR ONE FILE
        """
        cmd0 = "cfchecks --version auto %s" %item

        p0 = subprocess.Popen(
            cmd0, shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
            )

        out0, _ = p0.communicate()

        return out0

    def netcdf_test(self,args):
        """Check the CF COMPLIANCE FOR THE INPUT FILES
        """

        for ncfile in args:

            string = '\nCF compliance test for file:\n'
            string += str(args) + '\n'

            out0 = self.run_cfchecker(ncfile)

            string = str(out0).replace('\'',"").replace("\"","").split('\\n')
            if any('ERROR:' in e for e in string):
                err = [item.split('ERROR:')[-1].strip() for item in string if item.startswith('ERROR:')]
                errstring = ncfile.split('/')[-1] + ' has ' + str(len(err))
                errstring += ' error messages' + '\n'
                errstring += '\n'.join(err)
                string += errstring
                logging.error(string)
            else:
                string += 'NO ERRORS FOUND when checking against CF'
                print('NO ERRORS FOUND when checking against CF')
                logging.info(string)
                pass
            # We are only interested on errors but could be interesting for 
            # further versions of the dqc_checker to analyze the warnings 
            # and info messages
            # wrn = [item.split('WARN:')[-1].strip() for item in string if item.startswith('WARN:')]
            # inf = [item.split('INFO:')[-1].strip() for item in string if item.startswith('INFO:')]

    def grib_test(self,args):
        """Check the GRIB BASELINE FOR THE INPUT FILES
        """

        for gribfile in args:
            gcck.grib_cck_run(gribfile)

    def main(self):
        if self.type != None:

            initiate_logging(self.type, "LOG_TEST01_STANDARD_COMPLIANCE",
                             self.path)
            if self.type == 'CF':
                ncfiles = [item for item in self.files if item.endswith('.nc')]
                if ncfiles != []:

                    self.netcdf_test(ncfiles)
                else:
                    raise Exception('No NetCDF files in the given input')
            else:
                if self.type == 'grib':
                    gribfiles = [item for item in self.files if
                                 item.endswith('.grib')]
                    if gribfiles != []:
                        print("No CDM has been implemented yet for the "
                              "variables to be checked - listing keys present"
                              " in the files if grib2")
                        print(gribfiles)
                        self.grib_test(gribfiles)
                    else:
                        raise Exception('No GRIB files in the given input')
                else:
                    raise Exception('type not supported, type currently '
                                    'supported are grib and CF')
        else:
            raise Exception("No type selected, exiting")

if __name__ == "__main__":
    sys.exit()
